; #########################################################################
;
;   Soundz.asm
;
;   31/01/2004 - The Apprentice
;
; #########################################################################
    
    .486
    .model flat, stdcall                    ; FLAT memory model & STDCALL calling
    option casemap :none                    ; set code to case sensitive

; #########################################################################

    ; =================================
    ; Local include file for this file
    ; =================================
    include soundz.inc

; #########################################################################

    .const
        IDI_APPICON     equ 500             ; application icon ID (defined in rsrc.rc)

        IDW_MACARONI    equ 600             ; wave IDs (defined in rsrc.rc)
        IDW_OUTAHERE    equ 601
        IDW_WRONGBUT    equ 602
        
        IDPB_MACARONI   equ 4000            ; button IDs
        IDPB_WRONG      equ 4001

    .data
        szClassName         db "Soundz_Class", 0
        szTitlebarCaption   db "Soundz", 0
        szInstanceMsg       db "An instance of this application is already running", 0
        szNoSoundDevice     db "There's no sound device present to play the sound samples", 0

        szbtnMacaroni       db "Macaroni", 0    ; button captions
        szbtnWrong          db "Wrong", 0

        dwWinWidth          dd 200              ; main window default width
        dwWinHeight         dd 90               ; main window default height
        
        dwButtonYPos        dd 25               ; default Y position for PushButton
        dwButtonWidth       dd 75               ; default width for PushButton
        dwButtonHeight      dd 25               ; default height for PushButton

        dwSoundDevices      dd 0                ; number of wave sound devices detected

    .data?
        CommandLine         dd ?
        hInstance           dd ?
        hWnd                dd ?                ; main window handle
        hButtonMacaroni     dd ?                ; button handles. Needed to enable/disable the
        hButtonWrong        dd ?                ; buttons incase no soundcard is installed

; #########################################################################

    .code

start:
    invoke FindWindow, ADDR szClassName, ADDR szTitlebarCaption
    cmp    eax, 0                               ; if eax > 0 then an instance of
    je     continue                             ;   this program is already running

    invoke MessageBox, NULL, ADDR szInstanceMsg, ADDR szTitlebarCaption, MB_OK or MB_ICONEXCLAMATION
    jmp    bailout                              ; inform user and bail out

continue:
    invoke GetModuleHandle, NULL                ; provides the instance handle
    mov    hInstance, eax

    invoke GetCommandLine                       ; provides the command line address
    mov    CommandLine, eax

    invoke WinMain, hInstance, NULL, CommandLine, SW_SHOWDEFAULT

bailout:
    invoke ExitProcess, eax                     ; cleanup & return to operating system

; #########################################################################

WinMain proc hInst :DWORD, hPrevInst :DWORD, CmdLine :DWORD, CmdShow :DWORD

    LOCAL wc  :WNDCLASSEX
    LOCAL msg :MSG
    LOCAL Wtx :DWORD
    LOCAL Wty :DWORD

    mov    wc.cbSize, SIZEOF WNDCLASSEX
    mov    wc.style, CS_HREDRAW or CS_VREDRAW or CS_BYTEALIGNWINDOW
    mov    wc.lpfnWndProc, OFFSET WndProc
    mov    wc.cbClsExtra, NULL
    mov    wc.cbWndExtra, NULL
    m2m    wc.hInstance, hInst
    mov    wc.hbrBackground, COLOR_BTNFACE + 1
    mov    wc.lpszMenuName, NULL
    mov    wc.lpszClassName, OFFSET szClassName
    invoke LoadIcon, hInst, IDI_APPICON
    mov    wc.hIcon, eax
    invoke LoadCursor, NULL, IDC_ARROW
    mov    wc.hCursor, eax
    mov    wc.hIconSm, NULL

    invoke RegisterClassEx, ADDR wc

    invoke GetSystemMetrics, SM_CXSCREEN        ; get screen width in pixels
    invoke TopXY, dwWinWidth, eax
    mov    Wtx, eax

    invoke GetSystemMetrics, SM_CYSCREEN        ; get screen height in pixels
    invoke TopXY, dwWinHeight, eax
    mov    Wty, eax

    invoke CreateWindowEx, 0,                   ; use WS_EX_OVERLAPPEDWINDOW to draw a border
                           ADDR szClassName,
                           ADDR szTitlebarCaption,
                           WS_OVERLAPPEDWINDOW and not WS_MINIMIZEBOX and not \
                           WS_MAXIMIZEBOX and not WS_THICKFRAME,
                           Wtx, Wty, dwWinWidth, dwWinHeight,
                           NULL, NULL,
                           hInst, NULL

    mov    hWnd, eax                            ; save main window handle
    invoke ShowWindow, hWnd, SW_SHOWNORMAL      ; display the main window
    invoke UpdateWindow, hWnd                   ; update main window

    ; #################################################
    ; Here we detect if there's a sound device present
    ; that's capable of playing wave files
    ; #################################################
    invoke waveOutGetNumDevs                    ; get the number of devices
    mov    dwSoundDevices, eax                  ; save it
    cmp    eax, 0                               ; 0 means none present or error
    jne    startloop

    invoke EnableWindow, hButtonMacaroni, FALSE ; disable the buttons and inform user
    invoke EnableWindow, hButtonWrong, FALSE
    invoke MessageBox, NULL, ADDR szNoSoundDevice, ADDR szTitlebarCaption, MB_OK or MB_ICONEXCLAMATION

startloop:
    invoke GetMessage, ADDR msg, NULL, 0, 0
    cmp    eax, 0
    je     exitloop
    
    invoke TranslateMessage, ADDR msg
    invoke DispatchMessage, ADDR msg
    jmp    startloop

exitloop:
    return msg.wParam

WinMain endp

; #########################################################################

WndProc proc hWin :DWORD, uMsg :DWORD, wParam :DWORD, lParam :DWORD

    .if uMsg == WM_CREATE
        ;
        ; Create the Macaroni and Wrong button
        ;
        invoke PushButton, ADDR szbtnMacaroni, hWin, 20, dwButtonYPos, \
                           dwButtonWidth, dwButtonHeight, IDPB_MACARONI
        mov    hButtonMacaroni, eax
        invoke PushButton, ADDR szbtnWrong, hWin, 100, dwButtonYPos, \
                           dwButtonWidth, dwButtonHeight, IDPB_WRONG
        mov    hButtonWrong, eax
        jmp    endwndproc

    .elseif uMsg == WM_COMMAND
        ;
        ; Button commands
        ;
        cmp    wParam, IDPB_MACARONI            ; macaroni button clicked ?
        jne    nextbut
        invoke PlaySound, OFFSET IDW_MACARONI, hInstance, SND_RESOURCE or SND_ASYNC
        jmp    endwndproc
nextbut:
        cmp    wParam, IDPB_WRONG               ; wrong button clicked ?
        jne    endwndproc
        invoke  PlaySound, OFFSET IDW_WRONGBUT, hInstance, SND_RESOURCE or SND_ASYNC
        jmp    endwndproc

    .elseif uMsg == WM_CLOSE
        ;
        ; Skip the PlaySound function when there are no sound devices installed
        ;
        cmp    dwSoundDevices, 0                ; the number of sound devices was
        je     nosounddevices                   ; saved in dwSoundDevices
        invoke PlaySound, OFFSET IDW_OUTAHERE, hInstance, SND_RESOURCE or SND_SYNC

nosounddevices:
        invoke DestroyWindow, hWin              ; destroy main window
        jmp    endwndproc

    .elseif uMsg == WM_DESTROY
        ;
        ; DestroyWindow sends the WM_DESTROY message. Here's where it is intercepted.
        ;
        invoke PostQuitMessage, NULL
        return 0 
    .endif

    invoke DefWindowProc, hWin, uMsg, wParam, lParam
        ; =====================================================================
        ; Default window processing is done by the operating system for any
        ; message that is not processed by the application in the WndProc
        ; procedure. If the application requires other than default processing
        ; it executes the code when the message is trapped and returns ZERO
        ; to exit the WndProc procedure before the default window processing
        ; occurs with the call to DefWindowProc().
        ; =====================================================================

endwndproc:
    ret

WndProc endp

; ########################################################################

TopXY proc wDim:DWORD, sDim:DWORD

    ; =====================================================
    ; This procedure calculates the top X & Y co-ordinates
    ; for the CreateWindowEx call in the WinMain procedure
    ; =====================================================
    
    shr sDim, 1     ; divide screen dimension by 2
    shr wDim, 1     ; divide window dimension by 2
    mov eax, wDim   ; copy window dimension into eax
    sub sDim, eax   ; sub half win dimension from half screen dimension

    return sDim

TopXY endp

; ########################################################################

PushButton proc lpText :DWORD, hParent :DWORD, a :DWORD, b :DWORD, wd :DWORD, ht :DWORD, ID :DWORD

    LOCAL hButton :DWORD

    szText btnClass, "BUTTON"

    invoke CreateWindowEx,
            WS_EX_STATICEDGE,                   ; replace this with 0 if no border wanted
            ADDR btnClass,
            lpText,
            WS_CHILD or WS_VISIBLE,
            a, b, wd, ht,
            hParent, ID,
            hInstance, NULL

    mov    hButton, eax
    invoke GetStockObject, ANSI_VAR_FONT        ; GetStockObject in gdi32.inc
    invoke SendMessage, hButton, WM_SETFONT, eax, 0 ; eax holds the font handle

    return hButton                              ; return the newly created button's handle

PushButton endp

; ########################################################################

end start