; #########################################################################
;
;   soundz.inc
;
;	Include file for soundz.asm
;
;   (C)2004  Patrick Meersman aka The Apprentice
;
; #########################################################################

	; ==============================================
	; main include file with equates and structures
	; ==============================================

	include \masm32\include\windows.inc

	; ==============================================================
	; In MASM32, each include file created by the L2INC.EXE utility
	; has a matching library file. If you need functions from a
	; specific library, you use BOTH the include file and library
	; file for that library.
	; ==============================================================

	include \masm32\include\user32.inc
	include \masm32\include\kernel32.inc
	include \masm32\include\gdi32.inc
	include \masm32\include\winmm.inc

	includelib \masm32\lib\user32.lib
	includelib \masm32\lib\kernel32.lib
	includelib \masm32\lib\gdi32.lib
	includelib \masm32\lib\winmm.lib

; #########################################################################

	; =========================================================================
	; MACROS are a method of expanding text at assembly time. This allows the
	; programmer a tidy and convenient way of using COMMON blocks of code with
	; the capacity to use DIFFERENT parameters in each block.
	; =========================================================================

	; 1. szText
	; A macro to insert TEXT into the code section for convenient and 
	; more intuitive coding of functions that use byte data as text.

	szText MACRO Name, Text:VARARG
		LOCAL lbl
		jmp lbl
		Name db Text,0
		lbl:
	ENDM

    ; 2. m2m
    ; There is no mnemonic to copy from one memory location to another,
    ; this macro saves repeated coding of this process and is easier to
    ; read in complex code.

    m2m MACRO M1, M2
        push M2
        pop  M1
    ENDM

	; 3. return
    ; Every procedure MUST have a "ret" to return the instruction
    ; pointer EIP back to the next instruction after the call that
    ; branched to it. This macro puts a return value in eax and
    ; makes the "ret" instruction on one line. It is mainly used
    ; for clear coding in complex conditionals in large branching
    ; code such as the WndProc procedure.

	return MACRO arg
        mov eax, arg
        ret
	ENDM

; #########################################################################

	; =======================================================================
	; Prototypes are used in conjunction with the MASM "invoke" syntax for
	; checking the number and size of parameters passed to a procedure. This
	; improves the reliability of code that is written where errors in
	; parameters are caught and displayed at assembly time.
	; =======================================================================

	WinMain		PROTO :DWORD,:DWORD,:DWORD,:DWORD
    WndProc		PROTO :DWORD,:DWORD,:DWORD,:DWORD
    TopXY		PROTO :DWORD,:DWORD
	PushButton	PROTO :DWORD,:DWORD,:DWORD,:DWORD,:DWORD,:DWORD,:DWORD
	
; #########################################################################